// *********************************************************************
//   Universal Gateway Custom DPA Handler - Template 
// *********************************************************************
// File:    CustomDpaHandler.c
// Version: Revision: 1.02
// Date:    2016/08/16
//
// Revision history:
//   2016/07/28  Release for DPA v2.27 IQRF OS v3.08D
//   2016/08/16  UniGW Task function description updated
//
// *********************************************************************

// Online Universal gateway documentation: http://www.iqhome.org/products/uni-gw/
// Online DPA documentation http://www.iqrf.org/DpaTechGuide/

// Default IQRF include (modify the path according to your setup)
#include "include/IQRF_OS/IQRF.h"

// Uncomment to implement Custom DPA Handler for Coordinator
#define COORDINATOR_CUSTOM_HANDLER

// Default DPA header (modify the path according to your setup)
#include "include/DPA/DPA.h"
// Default Custom DPA Handler header (modify the path according to your setup)
#include "include/DPA/DPAcustomHandler.h"

//############################################################################################
// HWP ID number definition (Coordinator)
//############################################################################################
#define HWP_ID  0x0312  // Hardware profile ID
#define HWP_VER 0x0102  // Hardware profile version
//############################################################################################

//############################################################################################
// Configuration part of Universal Gateway Task Function:
//############################################################################################
// Uncomment next line to disable RF network traffic LED indication
// #define DISABLE_LEDindication
// -------------------------------------------------------------------------------------------
// Uncomment next line to disable initialization of unused pins
// #define DISABLE_initUnusedPins
//############################################################################################
 
// Universal Gateway Task Function header (modify the path according to your setup)
#include "include/UniGWtask.h"  // !!! DO NOT REMOVE THIS INCLUDE !!!

//############################################################################################

// Place for global variables shared among CustomDpaHandler() and other function, otherwise local [static] variables are recommended
// example: uns8 globalCounter;

// Must be the 1st defined function in the source code in order to be placed at the correct FLASH location!
//############################################################################################
bit CustomDpaHandler()
//############################################################################################
{
  // Handler presence mark
  clrwdt();

  //############################################################################################
  // UniGW Task Function - !!! DO NOT REMOVE THIS FUNCTION CALL!!!
  //############################################################################################
  // Function description:
  // ---------------------
  // Visualization of RF network traffic across LEDR and LEDG pins.
  //  Green: Sending data to RF network or local DPA command received.
  //  Red: Receiving data from RF network or transmitting response of local DPA command to gateway.
  //
  //  Note: Visualization of RF network traffic can be disabling with defining macro:
  //  DISABLE_LEDindication
  //  -> Example: #define DISABLE_LEDindication
  //
  // Set unused pins to output to reduce the power consumption of the module
  //  Note: Initialization of unused pins can be disabling with defining macro:
  //  DISABLE_initUnusedPins
  //  -> Example: #define DISABLE_initUnusedPins
  //                                         
  // Controls the RF network is busy status pin (Default pin is: PORTC.2)
  //  The pin is set to low after RF package was received from RF network.
  //  -> If the pin is low, than the RF network is still routing.
  //  The pin is set to high when the timing has expired.
  //  -> If the pin is high, than the RF network is ready to send data.
  //
  // Side effects:
  // -------------
  // The task controls the LEDR and LEDG pins with pulseLEDR() and pulseLEDG() OS function.
  // The task controls the PORTC.2 pin.
  // The delay is implemented with DpaTicks global variable. The value of DpaTicks variable is
  // modified by this function.
  // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  // !!! Do not use DpaTicks variable in your code !!!
  // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  // FSR0 and FSR1 registers are used as temporary variables.
  // Value of FSR0 and FSR1 registers will be destroyed.
  // 
  // Maximum program memory usage: 90 instruction
  //############################################################################################
  UniGWtask();
  //############################################################################################

  // Place for local static variables used only within CustomDpaHandler() among more events
  // example: static bit interruptOccured;

  // Detect DPA event to handle (unused event handlers can be commented out or even deleted)
  switch ( GetDpaEvent() )
  {
#ifdef DpaEvent_Interrupt
//	// -------------------------------------------------
//	case DpaEvent_Interrupt:
//	  // Do an extra quick background interrupt work
//	  // ! The time spent handling this event is critical.If there is no interrupt to handle return immediately otherwise keep the code as fast as possible.
//	  // ! Make sure the event is the 1st case in the main switch statement at the handler routine.This ensures that the event is handled as the 1st one.
//	  // ! It is desirable that this event is handled with immediate return even if it is not used by the custom handler because the Interrupt event is raised on every MCU interrupt and the empty return handler ensures the shortest possible interrupt routine response time.
//	  // ! Only global variables or local ones marked by static keyword can be used to allow reentrancy.
//	  // ! Make sure race condition does not occur when accessing those variables at other places.
//	  // ! Make sure( inspect.lst file generated by C compiler ) compiler does not create any hidden temporary local variable( occurs when using division, multiplication or bit shifts ) at the event handler code.The name of such variable is usually Cnumbercnt.
//	  // ! Do not call any OS functions except getINFx() and setINDFx().
//	  // ! Do not use any OS variables especially for writing access.
//	  // ! All above rules apply also to any other function being called from the event handler code, although calling any function from Interrupt event is not recommended because of additional MCU stack usage.
//
//	  return Carry;
#endif
//	  // -------------------------------------------------
//	case DpaEvent_Idle:
//	  // Do a quick background work when RF packet is not received
//	  break;

//	  // -------------------------------------------------
//	case DpaEvent_Reset:
//	  // Called after module is reset
//	  //goto DpaHandleReturnTRUE; // return TRUE only if you handle node bonding/unbonding
//	  break;

//	  // -------------------------------------------------
//	case DpaEvent_Init:
//	  // Do a one time initialization work before main loop starts
//	  break;

//	  // -------------------------------------------------
//	case DpaEvent_ReceiveDpaRequest:
//	  // Called after DPA request was received
//	  //goto DpaHandleReturnTRUE; // return TRUE to skip default processing
//	  break;

//	  // -------------------------------------------------
//	case DpaEvent_BeforeSendingDpaResponse:
//	  // Called before sending DPA response back to originator of DPA response
//	  break;

//	  // -------------------------------------------------
//	case DpaEvent_Notification:
//	  // Called after DPA request was processed and after DPA response was sent
//	  break;

//	  // -------------------------------------------------
//	case DpaEvent_AfterRouting:
//	  // Called after Notification and after routing of the DPA response was finished
//	  break;

#ifndef COORDINATOR_CUSTOM_HANDLER
//	  // -------------------------------------------------
//	case DpaEvent_FrcValue:
//	  // Called to get FRC value
//	  break;
#endif

#ifndef COORDINATOR_CUSTOM_HANDLER
//	  // -------------------------------------------------
//	case DpaEvent_FrcResponseTime:
//	  // Called to get FRC response time
//	  break;
#endif

#ifndef COORDINATOR_CUSTOM_HANDLER
//	  // -------------------------------------------------
//	case DpaEvent_BeforeSleep:
//	  // Called before going to sleep
//	  break;
#endif

#ifndef COORDINATOR_CUSTOM_HANDLER
//	  // -------------------------------------------------
//	case DpaEvent_AfterSleep:
//	  // Called after woken up after sleep
//	  break;
#endif

//	  // -------------------------------------------------
//	case DpaEvent_DisableInterrupts:
//	  // Called when device needs all hardware interrupts to be disabled (before Reset, Restart and RFPGM)
//	  break;

#ifdef COORDINATOR_CUSTOM_HANDLER
	  // -------------------------------------------------
//	case DpaEvent_ReceiveDpaResponse:
//	  // Called after DPA response was received at coordinator
//	  //goto DpaHandleReturnTRUE; // return TRUE to skip default processing
//	  break;
#endif

#ifdef COORDINATOR_CUSTOM_HANDLER
//	  // -------------------------------------------------
//	case DpaEvent_IFaceReceive:
//	  // Called after DPA request from interface master was received at coordinator
//	  //goto DpaHandleReturnTRUE; // return TRUE to skip default processing
//	  break;
#endif

//	  // -------------------------------------------------
//	case DpaEvent_PeerToPeer:
//	  // Called when peer-to-peer (non-networking) packet is received
//	  break;

//	  // -------------------------------------------------
//	case DpaEvent_AuthorizePreBonding:
//	  // Called when remote bonding is enabled and a node requests pre-bonding
//	  //goto DpaHandleReturnTRUE; // return TRUE to reject pre-bonding request
//	  break;

//	  // -------------------------------------------------
//	case DpaEvent_UserDpaValue:
//	  // Called when DPA is required to return User defined DPA value in the response
//	  break;

	  // -------------------------------------------------
	case DpaEvent_DpaRequest:
	  // Called to interpret DPA request for peripherals
	  // -------------------------------------------------
	  // Peripheral enumeration
	  if ( IsDpaEnumPeripheralsRequest() )
	  {
		_DpaMessage.EnumPeripheralsAnswer.UserPerNr = 0; // ?
		_DpaMessage.EnumPeripheralsAnswer.HWPID = HWP_ID; // ????
		_DpaMessage.EnumPeripheralsAnswer.HWPIDver = HWP_VER; // ????

DpaHandleReturnTRUE:
		return TRUE;
	  }
	  // -------------------------------------------------
	  // Get information about peripheral
	  else if ( IsDpaPeripheralInfoRequest() )
	  {
		if ( _PNUM == PNUM_USER + 0 ) // ?
		{
		  _DpaMessage.PeripheralInfoAnswer.PerT = 0; // PERIPHERAL_TYPE_?
		  _DpaMessage.PeripheralInfoAnswer.PerTE = 0; // PERIPHERAL_TYPE_EXTENDED_?
		  _DpaMessage.PeripheralInfoAnswer.Par1 = 0; // ?
		  _DpaMessage.PeripheralInfoAnswer.Par2 = 0; // ?
		  goto DpaHandleReturnTRUE;
		}

		break;
	  }
	  // -------------------------------------------------
	  else
	  {
		// Handle peripheral command
		if ( _PNUM == PNUM_USER + 0 ) // ?
		{
		  if ( _PCMD == 0 ) // ????
		  {
			goto DpaHandleReturnTRUE;
		  }
		}
	  }

	  break;
  }

DpaHandleReturnFALSE:
  return FALSE;
}

//############################################################################################
// Universal Gateway Task Function header; 2nd include (modify the path according to your setup)
#include "include/UniGWtask.h"  // !!! DO NOT REMOVE THIS INCLUDE !!!
//############################################################################################

//############################################################################################
// Default Custom DPA Handler header; 2nd include to implement Code bumper to detect too long code of the Custom DPA Handler (modify the path according to your setup) 
#include "include/DPA/DPAcustomHandler.h"
//############################################################################################
